<?php

namespace Drupal\page_cache_query_ignore\StackMiddleware;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\PageCache\RequestPolicyInterface;
use Drupal\Core\PageCache\ResponsePolicyInterface;
use Drupal\page_cache\StackMiddleware\PageCache;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Executes the page caching before the main kernel takes over the request.
 *
 * Ignore query parameters also.
 */
class PageCacheIgnore extends PageCache {

  /**
   * A config object for the page cache query parameters ignore.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * Ignored parameters.
   *
   * @var array
   */
  protected $ignoredParameters = [];

  /**
   * {@inheritdoc}
   */
  public function __construct(HttpKernelInterface $http_kernel, CacheBackendInterface $cache, RequestPolicyInterface $request_policy, ResponsePolicyInterface $response_policy, ConfigFactoryInterface $config_factory) {
    parent::__construct($http_kernel, $cache, $request_policy, $response_policy);
    $this->config = $config_factory->get('page_cache_query_ignore.settings');
  }

  /**
   * Get ignored query params.
   *
   * @return array
   *   Ignored params.
   */
  protected function getIgnoredParams() {
    if (empty($this->ignoredParameters)) {
      $this->ignoredParameters = $this->config->get('ignored_parameters');
    }
    return $this->ignoredParameters;
  }

  /**
   * {@inheritdoc}
   */
  protected function getCacheId(Request $request) {
    // Once a cache ID is determined for the request, reuse it for the duration
    // of the request. This ensures that when the cache is written, it is only
    // keyed on request data that was available when it was read. For example,
    // the request format might be NULL during cache lookup and then set during
    // routing, in which case we want to key on NULL during writing, since that
    // will be the value during lookups for subsequent requests.
    if (!isset($this->cid)) {
      $cid_parts = [
        $request->getSchemeAndHttpHost() . $this->clear($request->getRequestUri()),
        $request->getRequestFormat(NULL),
      ];
      $this->cid = implode(':', $cid_parts);
    }
    return $this->cid;
  }

  /**
   * Clear query string.
   *
   * @param string $value
   *   The value to cleanup.
   *
   * @return string
   *   The cleared value.
   */
  protected function clear($value) {
    $request_parts = parse_url($value);

    if (empty($request_parts['query'])) {
      return $value;
    }

    parse_str($request_parts['query'], $request_query);

    // Remove the query arguments that are excluded.
    $request_query = array_diff_key($request_query, array_flip($this->getIgnoredParams()));

    // Rebuild a new URI.
    $request_uri = $request_parts['path'];
    if (!empty($request_query)) {
      $request_uri .= '?' . http_build_query($request_query);
    }
    return $request_uri;
  }

}
